from ..struct_data import Struct
from ..db import DocumentDatabaseBackend
from ..util import CamelCaseConverter
from exceptions import NotFoundException


class SelfSavingStruct(Struct):

    __DOCUMENT_DB__ = DocumentDatabaseBackend()
    __PRIMARY_KEY__ = None

    def save(self):
        is_new = False
        if not self.verify_id():
            self.create_id()
            is_new = True
        self.pre_save()
        self.__DOCUMENT_DB__.set_doc(
                               self.__class__._get_document_name(),
                               self.to_struct()
                             )
        self.post_save(is_new)
        return self, is_new

    def delete(self):
        self.pre_delete()
        if self.verify_id():
            self.__DOCUMENT_DB__.delete_doc(
                                   self.__class__._get_document_name(),
                                   self.to_struct(),
                                   self.__class__
                                 )
        self.post_delete(self.verify_id())
        return self, self.verify_id()

    def pre_save(self):
        pass

    def post_save(self, saved):
        pass

    def pre_delete(self):
        pass

    def post_delete(self, deleted):
        pass

    def verify_id(self):
        return '_id' in self.__dict__

    def create_id(self):
        self['_id'] = hash(self)

    def __hash__(self):
        if '_id' in self.__dict__:
            return self._id
        h = 0
        for key, value in self.__dict__.iteritems():
            if key.startswith('_'):
                continue
            elif (isinstance(value, list) or
                  isinstance(value, dict)):
                value = str(value)
            h += hash(key) + hash(value)
        return h

    def __repr__(self):
        if self.__PRIMARY_KEY__ is not None:
            return "<%s - %s: %s>" % (
                     self.__class__.__name__,
                     self.__PRIMARY_KEY__,
                     self[self.__PRIMARY_KEY__]
            )
        return super(SelfSavingStruct, self).__repr__()

    @classmethod
    def _get_document_name(cls):
        doc_name = getattr(cls, '__DOCUMENT_NAME__', None)
        if doc_name is None:
            return CamelCaseConverter(cls.__name__).convert()
        return doc_name

    @classmethod
    def _not_found_exception(cls, params):
        raise NotFoundException(
          '%s not found for the following parameters %r' % (
                                                             cls.__name__,
                                                             params
                                                           )
        )

    @classmethod
    def all(cls, **params):
        documents = cls.__DOCUMENT_DB__.find_docs(cls._get_document_name(), params)
        documents_as_list = [x for x in documents]
        if not len(documents_as_list):
            cls._not_found_exception(params)
        return [cls(**document) for document in documents_as_list]

    @classmethod
    def get(cls, **params):
        item = cls.__DOCUMENT_DB__.get_doc(cls._get_document_name(), params)
        if item is not None:
            return cls(**item)
        cls._not_found_exception(params)

    @classmethod
    def teardown(cls):
        return cls.__DOCUMENT_DB__.teardown(cls._get_document_name())




from util.camel_case import CamelCaseConverterTest
from struct_data.struct_data import StructTest
from models.self_saving_struct import SelfSavingStructTest
from models.validating_struct import ValidatingStructTest
from db.documentdb import DocumentDatabaseTest, MemoryDatabaseBackendTest, MongoDatabaseBackendTest, CouchDatabaseBackendTest
from models.event_validating_struct import EventedModelTest



from unittest import TestCase

from mongomodels.db import DocumentDatabaseBackend, MemoryDatabaseBackend, CouchDatabaseBackend, MongoDatabaseBackend
from mongomodels.db.exceptions import NoDocumentDatabaseException


class DocumentDatabaseTest(TestCase):
    def test_not_implemented_will_raise_exception_on_any_call(self):
        not_implemented_db = DocumentDatabaseBackend()

        try:
            not_implemented_db.get_doc('foo', {})
            self.fail()
        except NoDocumentDatabaseException:
            pass

        try:
            not_implemented_db.teardown('test')
            self.fail()
        except NoDocumentDatabaseException:
            pass


class GenericBackendTest(object):
    def test_set_doc(self):
        self.db.set_doc('foo', {'_id':'1', 'bar':'baz'})
        self._assert_contents('foo', [{'_id':'1', 'bar':'baz'}])

    def test_get_doc(self):
        self.db.set_doc('foo', {'_id':'1', 'bar':'baz'})

        self.assertEquals({'_id':'1','bar':'baz'}, self.db.get_doc('foo', {}))
        self.assertEquals({'_id':'1','bar':'baz'}, self.db.get_doc('foo', {'_id':'1'}))

    def test_find_docs(self):
        self.db.set_doc('foo', {'_id':'1', 'bar':'baz'})

        self.assertEquals({'_id':'1','bar':'baz'}, self.db.find_docs('foo', {}).next())
        self.assertEquals({'_id':'1','bar':'baz'}, self.db.find_docs('foo', {'_id':'1'}).next())


class MongoDatabaseBackendTest(GenericBackendTest, TestCase):
    def setUp(self):
        self.db = MongoDatabaseBackend('localhost', 'test')

    def _assert_contents(self, coll_name, contents):
        collection = self.db.get_collection(coll_name)
        cursor = collection.find()
        self.assertEquals(1, cursor.count())
        self.assertEquals(contents[0], cursor.next())

    def tearDown(self):
        self.db.teardown('test')


class MemoryDatabaseBackendTest(GenericBackendTest, TestCase):
    def setUp(self):
        self.db = MemoryDatabaseBackend()

    def _assert_contents(self, coll_name, contents):
        self.assertEquals({coll_name:contents}, self.db.get_contents())

    def tearDown(self):
        self.db.clean()


class CouchDatabaseBackendTest(GenericBackendTest, TestCase):
    def setUp(self):
        self.db = CouchDatabaseBackend()

    def _assert_contents(self, coll_name, contents):
        collection = self.db.get_collection(coll_name)
        documents = collection.get_all()
        self.assertEquals(contents[0], collection.clean_document(documents.rows[0]))

    def assertEquals(self, expected, expectee, message=None):
        if isinstance(expectee, dict):
            if '_rev' in expectee:
                expectee.pop('_rev')
        super(CouchDatabaseBackendTest, self).assertEquals(expected, expectee, message)

    def tearDown(self):
        self.db.teardown('foo')



from copy import deepcopy

#This class is used as a base class for all objects that are persisted into MonngoDB
#it allows the object to be converted to and from a struct (e.g. dictionary, tuples, etc...)
#To create a persistent object that will be stored as a struct
#- Create a subclass of Struct
#- add a global variable 'classes' set it to a dictionary mapping name of attribute to class objec
#  for nested objects to be converted into a struct
#  - this will handle conversion of other objects (if the attribute is a collection then contents are
#    are assumed to be objects)
# for more details see the object bellow
# All persistent object should be able to be initialized with out any parameters

def is_struct(o):
    for c in o.__class__.__bases__:
        if c.__name__ == 'Struct':
            return True
    return o.__class__.__name__ == 'Struct'

def struct_doc(o):
    if is_struct(o):
        return deepcopy(o).to_struct()
    else:
        return o

class Struct(object):
    _match_all = '__ALL__'
    classes = {}

    def to_struct(self):
        d = dict(self.__dict__)
        if self.classes.has_key(self._match_all):
            keys = d.keys()
        else:
            keys = self.classes.keys()
            to_pop = []
            for k, v in d.iteritems():
                if k.startswith('__'):
                    to_pop.append(k)
                    continue
                if is_struct(v):
                    keys.append(k)
            for popping in to_pop:
                d.pop(popping)
        for name in keys:
            if d.get(name, None) != None:
                if d[name].__class__.__name__ == 'list':
                    d[name] = [o.to_struct() for o in d[name]]
                else:
                    d[name] = d[name].to_struct()
        return d


    def from_struct(self, d):
        self.__dict__.update(d)
        if self.classes.has_key(self._match_all):
            cls = self.classes[self._match_all]
            for name in self.__dict__.keys():
                a = getattr(self, name, None)
                if a != None:
                    if a.__class__.__name__  == 'list':
                        o = a.__class__()
                        for i in a:
                            no = cls()
                            no.from_struct(i)
                            o.append(no)
                    else:
                        o = cls()
                        o.from_struct(a)
                    setattr(self, name, o)
        else:
            for name, cls in self.classes.items():
                a = getattr(self, name, None)
                if a != None:
                    if a.__class__.__name__  == 'list':
                        o = a.__class__()
                        for i in a:
                            no = cls()
                            no.from_struct(i)
                            o.append(no)
                    else:
                        o = cls()
                        o.from_struct(a)
                    setattr(self, name, o)
        return self

    # make it look like a dict
    def __init__(self, *args, **kw):
        self.__dict__.__init__(*args, **kw)

    def __setitem__(self, key, value):
        self.__dict__.__setitem__(key, value)

    def __getitem__(self, key):
        return self.__dict__.__getitem__(key)

    def __delitem__(self, key):
        del self.__dict__[key]

    def __containes__(self, key):
        return self.__dict__.has_key(key)

    def __iter__(self):
        return self.__dict__.iterkeys()

    def get(self, key, default=None):
        return self.__dict__.get(key, default)

    def has_key(self, key):
        return self.__dict__.has_key(key)

    def setdefault(self, key, default):
        return self.__dict__.setdefault(key, default)

    def update(self, dict):
        return self.__dict__.update(dict)

    def iteritems(self):
        return self.__dict__.iteritems()

    def iterkeys(self):
        return self.__dict__.iterkeys()

    def keys(self):
        return self.__dict__.keys()

    def itervalues(self):
        return self.__dict__.itervalues()



from exceptions import ValidationException, NotFoundException
from self_saving_struct import SelfSavingStruct


class ValidatingStruct(SelfSavingStruct):
    __NULLABLES__ = []

    def __init__(self, **kwargs):
        super(ValidatingStruct, self).__init__(**kwargs)
        self.update(kwargs)

    def pre_save(self):
        self.validate()

    def validate(self):
        pass

    def _field_can_be_null(self, field_name):
        return (getattr(self, field_name, None) is None and 
                field_name in self.__NULLABLES__)

    def validate_field(self, field_name, validation_fn, nullable=True, exc_message='', *args):
        if self._field_can_be_null(field_name) and nullable:
            return
        if not validation_fn(self.__dict__.get(field_name, None), *args):
            raise ValidationException(exc_message)

    def validate_not_empty(self, field_name):
        self.validate_field(field_name, bool, nullable=False,
                            exc_message='%s is required' % (field_name,))

    def validate_existance(self, relationship_field, object_cls, nullable=False):
        if self._field_can_be_null(relationship_field) and nullable:
            return
        try:
            object_cls.get(_id=self[relationship_field])
        except NotFoundException:
            if nullable:
                return
            raise ValidationException('%s does not exist for id: %s' % (
                                        object_cls.__name__, self[relationship_field]))
        except KeyError:
            if nullable:
                return
            raise ValidationException('%s does not have the field %s' % (
                                        object_cls.__name__, relationship_field))

    def validate_in(self, field_name, valid_values):
        if self._field_can_be_null(field_name):
            return
        if self[field_name] not in valid_values:
            raise ValidationException(
            '''
            %s value for the %s field is not included in the allowed values (%s)
            ''' % (
              self[field_name], field_name, valid_values))

    def validate_type(self, field_name, expected_type):
        if self._field_can_be_null(field_name):
            return
        if not isinstance(self[field_name], expected_type):
            raise ValidationException('%s is not an instance of %s' % (
              field_name, str(expected_type)))




from event_handler import EventListener, EventThrower
from validating_struct import ValidatingStruct


class EventedValidatingStruct(ValidatingStruct, EventListener, EventThrower):
    def __init__(self, **kwargs):
        ValidatingStruct.__init__(self, **kwargs)
        EventListener.__init__(self)
        EventThrower.__init__(self)



from ..struct_data import is_struct
from exceptions import NoDocumentDatabaseException
import pymongo
import couchdb


class DocumentDatabaseBackend(object):
    def get_collection(self, coll_name):
        raise NoDocumentDatabaseException("A document database should be set for this model")

    def set_doc(self, coll_name, doc):
        collection = self.get_collection(coll_name)
        self._serialize(doc)
        collection.save(doc)

    def get_doc(self, coll_name, fdoc, struct_class=None):
        collection = self.get_collection(coll_name)
        doc = collection.find_one(fdoc)
        if doc and struct_class:
            return struct_class(**doc)
        return doc

    def find_docs(self, coll_name, fdoc=None, struct_class=None):
        collection = self.get_collection(coll_name)
        for doc in collection.find(fdoc):
            if doc and struct_class:
                yield struct_class(**doc)
            yield doc

    def delete_doc(self, coll_name, fdoc, struct_class = None):
        collection = self.get_collection(coll_name)
        doc = collection.remove(fdoc)
        if doc and struct_class:
            return struct_class(**doc)
        return doc

    def teardown(self, coll_name):
        raise NoDocumentDatabaseException("A document database should be set for this model")

    def _serialize(self, document):
        for k, v in document.items():
            if k != '_id' and is_struct(v):
                document[k] = v.to_struct()
            elif k == '$set' and isinstance(v, dict):
                for k2, v2 in v.items():
                    if k2 != '_id' and is_struct(v2):
                        v[k2] = v2.to_struct()


class MemoryDatabase(dict):
    pass


class Searchable(object):
    def _search(self, find_doc, document):
        for key, value in find_doc.iteritems():
            if document[key] != value:
                return False
        return True

    def find_one(self, find_doc):
        for document in self.get_all():
            if self._search(find_doc, self.clean_document(document)):
                return self.clean_document(document)

    def find(self, find_doc):
        for document in self.get_all():
            if self._search(find_doc, self.clean_document(document)):
                yield self.clean_document(document)

    def remove(self, find_doc):
        delete_index = None
        for index, document in enumerate(self.get_all()):
            if self._search(find_doc, document):
                delete_index = index
                break
        if delete_index is not None:
            return self.get_all().pop(delete_index)
        return None


class MemoryCollection(list, Searchable):
    def __init__(self, db, name):
        super(MemoryCollection, self).__init__()
        self.__DATABASE__ = db
        self.__NAME__ = name

    def get_all(self):
        return self

    def clean_document(self, document):
        return document

    def save(self, doc):
        to_update = None

        for index, document in enumerate(self.get_all()):
            if document['_id'] == doc['_id']:
                to_update = index
                break

        if to_update:
            self.get_all().pop(to_update)

        self.get_all().append(doc)

        self.__DATABASE__[self.__NAME__] = self.get_all()


class MemoryDatabaseBackend(DocumentDatabaseBackend):
    def __init__(self):
        super(MemoryDatabaseBackend, self).__init__()
        self.__CONTENT__ = MemoryDatabase()

    def get_collection(self, coll_name):
        return self.__CONTENT__.get(coll_name, MemoryCollection(self.__CONTENT__, coll_name))

    def get_contents(self):
        return self.__CONTENT__

    def teardown(self, coll_name):
        self.__CONTENT__[coll_name] = MemoryCollection(self.__CONTENT__, coll_name)
        return True

    def clean(self):
        self.__CONTENT__ = MemoryDatabase()


class MongoDatabaseBackend(DocumentDatabaseBackend):
    def __init__(self, mongo_uri, mongo_name):
        self.db = pymongo.Connection(mongo_uri)[mongo_name]

    def get_collection(self, coll_name):
        return self.db[coll_name]

    def teardown(self, coll_name):
        return bool(self.db[coll_name].remove())


class CouchDatabaseCollection(Searchable):
    def __init__(self, collection):
        self.__COLLECTION__ = collection

    def get_all(self):
        return self.__COLLECTION__.view('_all_docs', include_docs=True)

    def clean_document(self, document):
        def as_struct(document):
            clean = {}
            for k, v in document.iteritems():
                clean[k] = v
            return clean

        return as_struct(document.doc)

    def save(self, doc):
        _id = doc.pop('_id')
        self.__COLLECTION__[_id] = doc


class CouchDatabaseBackend(DocumentDatabaseBackend):
    def __init__(self, server=None):
        if server is None:
            self.server = couchdb.Server()
        else:
            self.server = couchdb.Server(server)

    def get_collection(self, coll_name):
        try:
            return CouchDatabaseCollection(self.server[coll_name])
        except couchdb.ResourceNotFound:
            return CouchDatabaseCollection(self.server.create(coll_name))

    def teardown(self, coll_name):
        try:
            del self.server[coll_name]
            return True
        except couchdb.ResourceNotFound:
            return False




import re


class CamelCaseConverter(object):

    first_cap_re = re.compile('(.)([A-Z][a-z]+)')
    all_cap_re = re.compile('([a-z0-9])([A-Z])')

    def __init__(self, text):
        self.text = text

    def convert(self):
            s1 = self.first_cap_re.sub(r'\1_\2', self.text)
            return self.all_cap_re.sub(r'\1_\2', s1).lower()



from unittest import TestCase
from mongomodels.models import EventedValidatingStruct
from event_handler.events import EventThrower
from mongomodels.db import MemoryDatabaseBackend


class MyEventedModel(EventedValidatingStruct):
    __DOCUMENT_DB__ = MemoryDatabaseBackend()

    def listen_something(self, *args, **kwargs):
        self.foo = 'bar'


class EventedModelTest(TestCase):
    def test_a_model_can_be_a_listener(self):
        listener = MyEventedModel(baz='bar')

        listener.on('something', listener.listen_something)

        thrower = EventThrower()
        thrower.add_listener(listener)

        self.assertTrue(getattr(listener, 'foo', None) is None)

        thrower.emit('something')

        self.assertEqual(listener.foo, 'bar')

        listener.save()
        self.assertEqual(listener.to_struct(), MyEventedModel.get().to_struct())

    def test_a_model_can_be_a_thrower(self):
        listener = MyEventedModel()

        listener.on('something', listener.listen_something)

        thrower = MyEventedModel(bar='baz')
        thrower.add_listener(listener)

        self.assertTrue(getattr(listener, 'foo', None) is None)

        thrower.emit('something')

        self.assertEqual(listener.foo, 'bar')

        thrower.save()
        self.assertEqual(thrower.to_struct(), MyEventedModel.get().to_struct())

    def tearDown(self):
        MyEventedModel.teardown()




from unittest import TestCase
from mongomodels.db import MemoryDatabaseBackend
from mongomodels.models import ValidatingStruct
from mongomodels.models.exceptions import ValidationException

class BaseValidatingModel(ValidatingStruct):
    __DOCUMENT_DB__ = MemoryDatabaseBackend()


class ValidatesNotEmptyFieldStruct(BaseValidatingModel):
    def validate(self):
        self.validate_not_empty('foo')


class CustomValidatedStruct(BaseValidatingModel):
    def validate(self):
        self.validate_field('foo', self.validate_foo)

    def validate_foo(self, value):
        return value == 'baz'


class ValidatesExistantRelationshipStruct(BaseValidatingModel):
    def __init__(self, nullable=False, **kwargs):
        self._nullable = nullable
        super(ValidatesExistantRelationshipStruct, self).__init__(**kwargs)

    def validate(self):
        self.validate_existance('father', ValidatesExistantRelationshipStruct, nullable=self._nullable)


class ValueInCollectionValidatingStruct(BaseValidatingModel):
    def validate(self):
        self.validate_in('foo', ['baz', 'boo'])


class TypeValidatingStruct(BaseValidatingModel):
    def validate(self):
        self.validate_type('meaning_of_life', int)


class WithoutNullableFieldStruct(BaseValidatingModel):
    def validate(self):
        self.validate_type('foo', int)


class WithNullableFieldStruct(WithoutNullableFieldStruct):
    __NULLABLES__ = ['foo']


class ValidatingStructTest(TestCase):
    def test_validates_not_empty(self):
        model_1 = ValidatesNotEmptyFieldStruct()

        self.assertRaises(ValidationException, model_1.save)

        model_1['foo'] = 'bar'

        model_1.save()

    def test_validates_existant_relation(self):
        father = ValidatesExistantRelationshipStruct(nullable=True, foo='bar')
        father.save()

        child = ValidatesExistantRelationshipStruct(father=father['_id'])
        child.save()

        fatherless_model = ValidatesExistantRelationshipStruct(bar='baz')
        self.assertRaises(ValidationException, fatherless_model.save)

    def test_custom_field_validation(self):
        model = CustomValidatedStruct(foo='bar')
        self.assertRaises(ValidationException, model.save)

        another_model = CustomValidatedStruct(foo='baz')
        another_model.save()

    def test_validates_value_in_collection(self):
        model = ValueInCollectionValidatingStruct(foo='bar')
        self.assertRaises(ValidationException, model.save)

        another_model = ValueInCollectionValidatingStruct(foo='baz')
        another_model.save()

    def test_validates_type(self):
        model = TypeValidatingStruct(meaning_of_life='bar')
        self.assertRaises(ValidationException, model.save)

        another_model = TypeValidatingStruct(meaning_of_life=42)
        another_model.save()

    def test_check_nullables(self):
        model = WithoutNullableFieldStruct(foo=None)
        self.assertRaises(ValidationException, model.save)

        another_model = WithNullableFieldStruct(foo=None)
        another_model.save()




from unittest import TestCase

from mongomodels.struct_data import is_struct, Struct

class MyStruct(Struct):
    pass

class StructTest(TestCase):
    def test_a_struct_is_a_struct_when_it_is_a_struct_or_a_child(self):
        self.assertFalse(is_struct(object()))
        self.assertTrue(is_struct(Struct()))
        self.assertTrue(is_struct(MyStruct()))

    def test_a_struct_can_be_accessed_as_a_dict_or_as_an_object(self):
        a_struct = Struct(foo='bar')

        self.assertEqual('bar', a_struct.foo)
        self.assertEqual('bar', a_struct['foo'])

    def test_a_struct_can_be_serialized_as_a_dict(self):
        a_struct = Struct(foo='bar')

        self.assertEqual({'foo':'bar'}, a_struct.to_struct())

    def test_a_struct_can_serialize_nested_structs(self):
        another_struct = Struct(baz='blah')
        a_struct = Struct(foo='bar', something=another_struct)

        self.assertEqual(
          {'foo':'bar', 'something': {'baz':'blah'}},
          a_struct.to_struct()
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

setup(
    name="mongomodels",
    version="0.6.0",
    description="A simple ODM for MongoDB",
    author="David Litvak",
    author_email = "david.litvakb@gmail.com",
    license = "GPL v3",
    keywords = "MongoDB ODM Database",
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
      'pymongo',
      'event_handler'
    ],
    url='http://github.com/dlitvakb',
)



from unittest import TestCase

from mongomodels.util import CamelCaseConverter

class CamelCaseConverterTest(TestCase):
    def assertConvertion(self, expectee, to_convert):
        self.assertEqual(expectee, CamelCaseConverter(to_convert).convert())

    def test_capitalized_to_lower(self):
        self.assertConvertion('hello', 'Hello')

    def test_one_lower_word_stays_the_same(self):
        self.assertConvertion('hello', 'hello')

    def test_camel_to_snake_if_no_consecutive_capitals(self):
        self.assertConvertion('hello_world', 'HelloWorld')

    def test_camel_to_snake_with_consecutive_capitals(self):
        self.assertConvertion('hello_world', 'HELLOWorld')



from unittest import TestCase

from mongomodels.models import SelfSavingStruct
from mongomodels.models.exceptions import NotFoundException
from mongomodels.db.exceptions import NoDocumentDatabaseException
from mongomodels.db import MemoryDatabaseBackend

class Model(SelfSavingStruct):
    __DOCUMENT_DB__ = MemoryDatabaseBackend()


class OtherModel(Model):
    __DOCUMENT_NAME__ = 'model'


class SelfSavingStructTest(TestCase):
    def test_when_no_database_is_assigned_should_raise_exception_on_action(self):
        document = SelfSavingStruct()
        self.assertRaises(NoDocumentDatabaseException, document.save)

    def test_when_database_is_assigned_should_save_properly(self):
        model = Model(foo='bar')
        model.save()

        self.assertEqual(model.to_struct(), Model.get().to_struct())

    def test_all(self):
        model = Model(foo='bar')
        model.save()

        self.assertEqual(model.to_struct(), Model.all()[0].to_struct())

    def test_when_collection_is_empty_should_raise_an_exception(self):
        self.assertRaises(NotFoundException, Model.get)
        self.assertRaises(NotFoundException, Model.all)

    def test_can_search_for_individual_objects(self):
        model_1 = Model(foo='bar')
        model_1.save()

        model_2 = Model(foo='baz')
        model_2.save()

        self.assertEqual(model_1.to_struct(), Model.get(foo='bar').to_struct())
        self.assertEqual(model_2.to_struct(), Model.get(foo='baz').to_struct())

    def test_can_search_for_multiple_objects(self):
        model_1 = Model(foo='bar')
        model_1.save()

        model_2 = Model(foo='bar', blah='bleh')
        model_2.save()

        models = [document.to_struct() for document in Model.all(foo='bar')]

        self.assertTrue(model_1.to_struct() != model_2.to_struct())

        self.assertTrue(model_1.to_struct() in models)
        self.assertTrue(model_2.to_struct() in models)

    def test_can_delete_a_document(self):
        model = Model(foo='bar')
        model.save()

        self.assertEqual(model.to_struct(), Model.get().to_struct())

        model.delete()

        self.assertRaises(NotFoundException, Model.get)

    def test_teardown_restarts_the_entire_collection(self):
        model = Model(foo='bar')
        model.save()

        self.assertEqual(model.to_struct(), Model.get().to_struct())

        Model.teardown()
        self.assertRaises(NotFoundException, Model.get)

    def test_two_models_can_point_to_the_same_collections(self):
        model_1 = Model(foo='bar')
        model_1.save()

        self.assertEqual(1, len(Model.all()))

        model_2 = OtherModel(bar='baz')
        model_2.save()

        self.assertEqual(2, len(Model.all()))
        self.assertEqual(2, len(OtherModel.all()))

    def test_a_model_can_have_a_list_as_a_field_value(self):
        model = Model(foo=['blah', 'bleh'])
        model.save()

        self.assertTrue(isinstance(Model.get().foo, list))

    def tearDown(self):
        Model.teardown()
